package com.widget.imagevideobanner.utils;

import android.content.ContentResolver;
import android.content.Context;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.text.TextUtils;

import com.widget.imagevideobanner.bean.MediaBean;
import com.widget.imagevideobanner.log.BLogUtil;

import java.io.File;
import java.io.FileOutputStream;

import wseemann.media.FFmpegMediaMetadataRetriever;

public class ResourceUtils {


    /**
     * drawable 转成 uri
     *
     * @param context
     * @param resId
     * @return
     */
    public static String imageTranslateUri(Context context, int resId) {
        Resources r = context.getResources();
        Uri uri = Uri.parse(ContentResolver.SCHEME_ANDROID_RESOURCE + "://"
                + r.getResourcePackageName(resId) + "/"
                + r.getResourceTypeName(resId) + "/"
                + r.getResourceEntryName(resId));

        return uri.toString();
    }



    /**
     * 解析宽高
     */
    public static void resolveVideo(MediaBean mediaBean) {

        BLogUtil.log("截取视频第一帧开始");

        if (TextUtils.isEmpty(mediaBean.getUrl())) {
            return;
        }

        File tempFile = new File(mediaBean.getUrl());
        if (!tempFile.exists()) {
           return;
        }

        //信息是全的
        if (!TextUtils.isEmpty(mediaBean.getCover())
                && mediaBean.getWidth() != 0
                && mediaBean.getHeight() != 0) {
            return;
        }

        //配置了缩略图 但是没配置宽高
        String cover = !TextUtils.isEmpty(mediaBean.getCover())?mediaBean.getCover() :
                mediaBean.getUrl().substring(0, mediaBean.getUrl().lastIndexOf(".")) + ".png";

        File file = new File(cover);
        if (file.exists()) {
            mediaBean.setCover(cover);
            if (mediaBean.getWidth() == 0 || mediaBean.getHeight() == 0) {
                BitmapFactory.Options options = new BitmapFactory.Options();
                options.inJustDecodeBounds = true;
                BitmapFactory.decodeFile(cover, options);
                int outWidth = options.outWidth;
                int outHeight = options.outHeight;
                mediaBean.setWidth(outWidth);
                mediaBean.setHeight(outHeight);
            }
            return;
        }

        FFmpegMediaMetadataRetriever mmr = null;
        Bitmap bmp = null;
        try {
            mmr = new FFmpegMediaMetadataRetriever();
            mmr.setDataSource(mediaBean.getUrl());
            mmr.extractMetadata(FFmpegMediaMetadataRetriever.METADATA_KEY_ALBUM);
            mmr.extractMetadata(FFmpegMediaMetadataRetriever.METADATA_KEY_ARTIST);
            bmp = mmr.getFrameAtTime();

            FileOutputStream out = new FileOutputStream(cover);
            bmp.compress(Bitmap.CompressFormat.PNG, 100, out);
            mediaBean.setCover(cover);
            mediaBean.setWidth(bmp.getWidth());
            mediaBean.setHeight(bmp.getHeight());

            BLogUtil.log("截取视频第一帧结束");

        } catch (Exception e) {
            e.printStackTrace();
            BLogUtil.log("截取视频第一帧异常"+ThrowableUtils.getFullStackTrace(e));
        } finally {
            if (null != bmp) bmp.recycle();
            if (null != mmr) mmr.release();
        }
    }

}
