package com.widget.imagevideobanner.view;

import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.view.PagerAdapter;
import android.text.TextUtils;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;

import com.alibaba.fastjson.JSON;
import com.blankj.utilcode.util.FileUtils;
import com.widget.imagevideobanner.R;
import com.widget.imagevideobanner.bean.MediaBean;
import com.widget.imagevideobanner.log.BLogUtil;
import com.widget.imagevideobanner.player.IPlayTarget;
import com.widget.imagevideobanner.utils.ResourceUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import static com.widget.imagevideobanner.bean.MediaBean.TYPE_VIDEO;

/**
 * 图片+视频
 * 混播控件
 */
public class ImageVideoBanner extends FrameLayout implements ViewPager.OnPageChangeListener, OnActionFinishListener {

    private final static String HTTP = "http";
    private ViewPager viewPager;
    private final List<View> viewArrayList = new ArrayList<>();
    private final List<MediaBean> mediaBeanList = new ArrayList<>();
    private ViewsPagerAdapter viewsPagerAdapter;
    private int curIndex;
    private final ExecutorService sExecutor = Executors.newFixedThreadPool(3);

    private final Handler handler = new Handler(Looper.getMainLooper(), new Handler.Callback() {
        @Override
        public boolean handleMessage(Message msg) {
            if (msg.what == 1) {
                BLogUtil.log(JSON.toJSONString(mediaBeanList));
                for (MediaBean mediaBean : mediaBeanList) {
                    if (!TextUtils.isEmpty(mediaBean.getUrl())) {
                        if (mediaBean.getType() == TYPE_VIDEO) {
                            viewArrayList.add(new VideoSubView(getContext(), mediaBean, ImageVideoBanner.this));
                        } else {
                            viewArrayList.add(new ImageSubView(getContext(), mediaBean, ImageVideoBanner.this));
                        }
                    }
                }
                if (viewArrayList.size() > 0) {
                    viewsPagerAdapter.notifyDataSetChanged();
                    //onAttachedToWindow 执行的比较早
                    ((IPlayTarget) viewArrayList.get(0)).onActive();
                }
            }
            return false;
        }
    });

    private ImageVideoBanner(@NonNull Context context) {
        super(context);
        initView();
    }

    public ImageVideoBanner(@NonNull Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
        initView();
    }

    private void initView() {
        View view = LayoutInflater.from(getContext()).inflate(R.layout.banner_imge_video, this, true);
        viewPager = view.findViewById(R.id.view_pager);
        viewsPagerAdapter = new ViewsPagerAdapter(viewArrayList);
        viewPager.setAdapter(viewsPagerAdapter);
        viewPager.setOnPageChangeListener(this);
    }

    public void bindData(List<MediaBean> beans) {

        final List<MediaBean> realMedias = filterNoExist(beans);

        if (null == realMedias || realMedias.size() == 0) {
            BLogUtil.log("bindData数据是空！！！");
            viewArrayList.clear();
            viewArrayList.add(new VideoSubView(getContext(), MediaBean.getDefaultMedia(getContext()), this));
            viewsPagerAdapter.notifyDataSetChanged();
            return;
        }

        BLogUtil.log("bindData轮播数据：" + JSON.toJSONString(realMedias));

        sExecutor.execute(new Runnable() {
            @Override
            public void run() {
                resolveMedia(realMedias);
                handler.sendEmptyMessage(1);
            }
        });
    }


    /**
     * 资源有更新
     */
    public void reflesh(List<MediaBean> beans) {
        List<MediaBean> realMedias = filterNoExist(beans);

        if (null == realMedias || realMedias.size() == 0) return;

        if (mediaBeanList.size() == 0 || mediaBeanList.size() != realMedias.size()) {
            bindData(realMedias);
        }
    }

    /**
     * 排除不存在的视频
     */
    private List<MediaBean> filterNoExist(List<MediaBean> beans) {
        if (beans == null || beans.size() <= 0) return null;
        List<MediaBean> list = new ArrayList<>();
        for (MediaBean mediaBean : beans) {
            if (!TextUtils.isEmpty(mediaBean.getUrl())) {
                if (mediaBean.getType() == TYPE_VIDEO) {
                    if (!mediaBean.getUrl().startsWith(HTTP) && FileUtils.isFileExists(mediaBean.getUrl())) {
                        list.add(mediaBean);
                    }
                } else {
                    list.add(mediaBean);
                }
            }
        }
        return list;
    }

    /**
     * 截取首帧 宽高【耗时】
     */
    private void resolveMedia(List<MediaBean> beans) {
        mediaBeanList.clear();
        mediaBeanList.addAll(beans);
        for (MediaBean mediaBean : mediaBeanList) {
            if (!TextUtils.isEmpty(mediaBean.getUrl()) && mediaBean.getType() == TYPE_VIDEO) {
                ResourceUtils.resolveVideo(mediaBean);
            }
        }
    }

    @Override
    protected void onWindowVisibilityChanged(int visibility) {
        super.onWindowVisibilityChanged(visibility);
        if (viewArrayList.size() > 0) {
            if(visibility == VISIBLE){
                BLogUtil.log("onWindowVisibilityChanged 视图可见，开启广告");
                IPlayTarget iPlayTarget = (IPlayTarget) viewArrayList.get(curIndex);
                iPlayTarget.onActive();
            }else{
                BLogUtil.log("onWindowVisibilityChanged 视图隐藏，暂停广告");
                IPlayTarget iPlayTarget = (IPlayTarget) viewArrayList.get(curIndex);
                iPlayTarget.inActive();
            }
        }
    }

    @Override
    public void onPageScrolled(int position, float positionOffset, int positionOffsetPixels) {
    }

    @Override
    public void onPageSelected(int position) {
        IPlayTarget iPlayTarget = (IPlayTarget) viewArrayList.get(curIndex);
        iPlayTarget.onActive();
    }

    @Override
    public void next() {
        curIndex = curIndex + 1;
        if (curIndex >= viewArrayList.size()) {
            curIndex = 0;
        }
        viewPager.setCurrentItem(curIndex);
    }

    @Override
    public void onPageScrollStateChanged(int state) {
    }

    private static final class ViewsPagerAdapter extends PagerAdapter {

        private final List<View> mList;

        public ViewsPagerAdapter(List<View> list) {
            this.mList = list;
        }

        @Override
        public int getCount() {
            return mList.size();
        }

        @Override
        public boolean isViewFromObject(@NonNull View view, @NonNull Object object) {
            return view == object;
        }

        @NonNull
        @Override
        public View instantiateItem(ViewGroup container, int position) {
            container.addView(mList.get(position));
            return mList.get(position);
        }

        @Override
        public void destroyItem(ViewGroup container, int position, @NonNull Object object) {
            container.removeView(mList.get(position));
        }
    }


}
